from ekp_sdk.services import CacheService
import os
from google.oauth2 import service_account
from apiclient import discovery


class GoogleSheetsClient:

    def __init__(
        self,
        credentials_file: str
        ):
        scopes = [
            "https://www.googleapis.com/auth/drive",
            "https://www.googleapis.com/auth/drive.file",
            "https://www.googleapis.com/auth/spreadsheets"
        ]
        secret_file = os.path.join(os.getcwd(), credentials_file)
        credentials = service_account.Credentials.from_service_account_file(
            secret_file,
            scopes=scopes
        )
        self.service = discovery.build('sheets', 'v4', credentials=credentials)

    def get_range(self, sheet_id, range):
        sheet = self.service.spreadsheets()

        result = sheet.values().get(
            spreadsheetId=sheet_id,
            range=range
        ).execute()

        return result.get('values', [])



from ekp_sdk.util.clean_null_terms import clean_null_terms


def Paragraphs(children, class_name=None):
    return {
        "_type": "Paragraphs",
        "props": clean_null_terms({
            "class_name": class_name,
            "children": children,
        })
    }


def Fragment(children, class_name=None):
    return {
        "_type": "Fragment",
        "props": clean_null_terms({
            "className": class_name,
            "children": children,
        })
    }


def Span(content, class_name=None, when=None):
    return {
        "_type": "Span",
        "props": clean_null_terms({
            "className": class_name,
            "content": content,
            "when": when
        })
    }


def Container(children, class_name=None, context=None):
    return {
        "_type": "Container",
        "props": clean_null_terms({
            "className": class_name,
            "children": children,
            "context": context
        })
    }


def Row(children, class_name=None):
    return {
        "_type": "Row",
        "props": clean_null_terms({
            "className": class_name,
            "children": children
        })
    }


def Div(children=None, class_name=None, style=None, when=None, context=None):
    return {
        "_type": "Div",
        "props": clean_null_terms({
            "className": class_name,
            "children": children or [],
            "style": style,
            "when": when,
            "context": context
        })
    }

def Timeline(events=None, title=None, content=None, style=None, class_name = None):
    return {
        "_type": "Timeline",
        "props": clean_null_terms({
            "className": class_name,
            "events": events or [],
            "title": title,
            "content": content,
            "style": style,
        })
    }
    
def Ul(items=None, style=None, class_name=None):
    return {
        "_type": "Ul",
        "props": clean_null_terms({
            "className": class_name,
            "style": style,
            "items": items or [],
        })
    }
    
    

def Col(class_name=None, children=None, when=None):
    return {
        "_type": "Col",
        "props": clean_null_terms({
            "className": class_name,
            "children": children or [],
            "when": when
        })
    }


def Icon(name, class_name=None, size=None):
    return {
        "_type": "Icon",
        "props": clean_null_terms({
            "className": class_name,
            "name": name,
            "size": size
        })
    }


def Datatable(
    data,
    columns,
    busy_when=None,
    class_name=None,
    default_sort_asc=None,
    default_sort_field_id=None,
    default_view=None,
    disable_list_view=None,
    filters=None,
    grid_view=None,
    on_row_clicked=None,
    pagination=None,
    pagination_per_page=None,
    search_hint=None,
    show_export=None,
    show_last_updated=None,
    card=None
):
    return {
        "_type": "Datatable",
        "props": clean_null_terms({
            "busyWhen": busy_when,
            "card": card,
            "className": class_name,
            "columns": columns,
            "data": data,
            "defaultSortAsc": default_sort_asc,
            "defaultSortFieldId": default_sort_field_id,
            "defaultView": default_view,
            "disableListView": disable_list_view,
            "filters": filters,
            "gridView": grid_view,
            "onRowClicked": on_row_clicked,
            "pagination": pagination,
            "paginationPerPage": pagination_per_page,
            "searchHint": search_hint,
            "showExport": show_export,
            "showLastUpdated": show_last_updated,
        })
    }


def Hr(class_name=None):
    return {
        "_type": "Hr",
        "props": {
            "className": class_name
        }
    }


def Badge(color, children, class_name=None):
    return {
        "_type": "Badge",
        "props": clean_null_terms({
            "children": children,
            "className": class_name,
            "color": color,
        })
    }


def Column(
    id,
    cell=None,
    format=None,
    grow=None,
    omit=None,
    min_width=None,
    right=None,
    searchable=None,
    sortable=None,
    title=None,
    value=None,
    width=None,
):
    return clean_null_terms({
        "cell": cell,
        "format": format,
        "grow": grow,
        "id": id,
        "minWidth": min_width,
        "omit": omit,
        "right": right,
        "searchable": searchable,
        "sortable": sortable,
        "title": title,
        "value": value,
        "width": width,
    })


def Card(children=None, class_name=None):
    return {
        "_type": "Card",
        "props": clean_null_terms({
            "children": children,
            "className": class_name
        })
    }


def Select(label, name, options, min_width=None, class_name=None):
    return {
        "_type": "Select",
        "props": clean_null_terms({
            "className": class_name,
            "label": label,
            "name": name,
            "options": options,
            "minWidth": min_width,
        })
    }


def Button(
    busy_when=None,
    class_name=None,
    color=None,
    icon=None,
    is_submit=None,
    label=None,
    on_click=None,
    size=None,
    tooltip=None
):
    return {
        "_type": "Button",
        "props": clean_null_terms({
            "busyWhen": busy_when,
            "className": class_name,
            "color": color,
            "icon": icon,
            "isSubmit": is_submit,
            "label": label,
            "onClick": on_click,
            "size": size,
            "tooltip": tooltip,
        })
    }


def collection(collectionName):
    return collectionName


def documents(collectionName):
    return f'$["{collection(collectionName)}"].*'


def is_busy(collection):
    return f'$.busy[?(@.id=="{collection}")]'


def format_currency(rpc, symbol):
    return {
        "method": "formatCurrency",
        "params": [rpc, symbol]
    }


def format_age(value):
    return {
        "method": "formatAge",
        "params": [value]
    }


def format_datetime(value):
    return {
        "method": "formatDatetime",
        "params": [value]
    }


def format_template(template, values):
    return {
        "method": "formatTemplate",
        "params": [template, values]
    }


def switch_case(on, cases):
    return {
        "method": "switchCase",
        "params": [on, cases]
    }


def json_array(values):
    return {
        "method": "jsonArray",
        "params": [values]
    }


def ekp_map(source, projection):
    return {
        "method": "map",
        "params": [source, projection]
    }


def sort_by(source, comparator):
    return {
        "method": "sortBy",
        "params": [source, comparator]
    }


def format_percent(value):
    return {
        "method": "formatPercent",
        "params": [value]
    }


def Image(src, style=None, class_name=None):
    return {
        "_type": "Image",
        "props": clean_null_terms({
            "src": src,
            "style": style,
            "className": class_name
        })
    }


def Avatar(icon, size=None, color=None):
    return {
        "_type": "Avatar",
        "props": clean_null_terms({
            "color": color,
            "icon": icon,
            "size": size
        })
    }

def Alert(content, header=None, icon_name=None, class_name=None, style=None, when=None):
    return {
        "_type": "Alert",
        "props": clean_null_terms({
            "content": content,
            "header": header,
            "iconName": icon_name,
            "className": class_name,
            "style": style,
            "when": when,
        })
    }
    



from typing import TypedDict


class MoralisNftDto(TypedDict, total=False):
    '''
    :example
    {
      "token_address": "0x05f0d89931eb06d4596af209de4a9779cef73cde",
      "token_id": "20300",
      "owner_of": "0x553a463f365c74eda00b7e5aaf080b066d4ca03c",
      "block_number": "17996141",
      "block_number_minted": "17996141",
      "token_hash": "ebe82b4820e228db95660090c8572adc",
      "amount": "1",
      "contract_type": "ERC721",
      "name": "MetaBombHero",
      "symbol": "MBH",
      "token_uri": "https://metabomb.io/v1/hero/20300",
      "metadata": null,
      "synced_at": "2022-05-21T13:50:01.211Z",
      "last_token_uri_sync": "2022-05-21T13:50:00.360Z",
      "last_metadata_sync": "2022-05-21T13:50:01.211Z"
    }
    '''

    token_address: str
    token_id: str
    owner_of: str
    block_number: str
    block_number_minted: str
    token_hash: str
    amount: str
    contract_type: str
    name: str
    symbol: str
    token_uri: str
    metadata: str
    synced_at: str
    last_token_uri_sync: str
    last_metadata_sync: str


from typing import List
from ekp_sdk.dto.moralis_nft_dto import MoralisNftDto
from ekp_sdk.services.limiter import Limiter
from ekp_sdk.services.rest_client import RestClient


class MoralisApiService:
    def __init__(
        self,
        api_key: str,
        rest_client: RestClient
    ):
        self.base_url = "https://deep-index.moralis.io/api/v2"
        self.api_key = api_key
        self.rest_client = rest_client
        self.limiter = Limiter(250, 4)

    # -----------------------------------------------------------------

    async def get_token_usd_price(
        self,
        chain: str,
        address: str,
        to_block_number: int = None
    ):

        url = f"{self.base_url}/erc20/{address}/price?chain={chain}"

        if to_block_number:
            url += f"&to_block={to_block_number}"
        
        def handle_response(data, text):
            if "usdPrice" in data:
                return data["usdPrice"]
            
            return 0
            
        result = await self.__get(
            url, 
            handle_response, 
            allowed_response_codes = [200, 400]
        )

        return result
    
    # -----------------------------------------------------------------

    async def get_token_metadata(
        self,
        chain: str,
        address: str,
    ):

        url = f"{self.base_url}/erc20/metadata?chain={chain}&addresses={address}"

        result = await self.__get(url, fn=lambda data,text: data[0])

        return result
    # -----------------------------------------------------------------

    async def get_nfts_by_owner_and_token_address(
        self,
        owner_address: str,
        token_address: str,
        chain: str
    ) -> List[MoralisNftDto]:

        url = f"{self.base_url}/{owner_address}/nft/{token_address}?chain={chain}&format=decimal"

        result = await self.__get(url)

        return result

    # -----------------------------------------------------------------

    async def __get(self, url, fn=lambda data, text: data["result"], allowed_response_codes = [200]):
        headers = {"X-API-Key": self.api_key}

        result = await self.rest_client.get(
            url,
            fn,
            limiter=self.limiter,
            headers=headers,
            allowed_response_codes=allowed_response_codes
        )

        return result



from ast import literal_eval

from ekp_sdk.domain.log import Log
from ekp_sdk.dto.etherscan_log_dto import EtherscanLogDto
from ekp_sdk.dto.web3_log_dto import Web3LogDto


class BaseMapperService:

    def map_hex_to_int(self, hex: str):
        if hex is None:
            return None

        if isinstance(hex, int):
            return hex

        if not isinstance(hex, str):
            return None

        if hex.startswith("0x"):
            return literal_eval(hex)

        return int(hex, 16)

    def map_etherscan_log_dto_to_domain(self, dto: EtherscanLogDto) -> Log:
        log: Log = {
            'address': dto['address'],
            'block_number': self.map_hex_to_int(dto['blockNumber']),
            'data': dto['data'],
            'gas_price': self.map_hex_to_int(dto['gasPrice']),
            'gas_used': self.map_hex_to_int(dto['gasUsed']),
            'log_index': self.map_hex_to_int(dto['logIndex']),
            'timestamp': self.map_hex_to_int(dto['timeStamp']),
            'topics': dto['topics'],
            'transaction_hash': dto['transactionHash'],
            'transaction_index': self.map_hex_to_int(dto['transactionIndex']),
        }

        return log

    def map_web3_log_dto_to_domain(self, dto: Web3LogDto) -> Log:
        log: Log = {
            'address': dto['address'],
            'block_number': dto['blockNumber'],
            'data': dto['data'],
            'log_index': dto['logIndex'],
            'topics': dto['topics'],
            'transaction_hash': dto['transactionHash'],
            'transaction_index': dto['transactionIndex'],
        }

        return log
